program readsaf91

!     Description:
!
!     Read 91-level diverse profile dataset from ECMWF
!     Owner:
!     Frederic Chevallier
!
!     History:
!     Version      Date        Comment
!     1            22/11/2006  Frederic Chevallier, LSCE
!     Code description:
!       Language:              Fortran 90.
!       Software Standards:    "European Standards for Writing and Documenting
!                              Exchangeable Fortran 90 code".

implicit none

integer, parameter      :: nlev = 91 ! number of vertical levels

real, dimension(nlev)   :: temp, hum, ozo, cc, clw, ciw
real, dimension(nlev)   :: rain, snow, w, pap
real, dimension(nlev+1) :: paph
real                    :: lnpsurf, psurf, z0, tsurf, t2m, td2m, hum2m, u10, v10
real                    :: stratrsrf, convrsrf, snowsurf, elev
real                    :: lsm, lat, long
real                    :: alb, sr, tvl, tvh, cvl, cvh, seaice
real                    :: asn, rsn, tsn, sd, stl1, stl2, stl3, stl4
real                    :: swvl1, swvl2, swvl3, swvl4, istl1, istl2, istl3, istl4
integer                 :: year, month, day, step, gpoint, ind, i
character(len=10)       :: cvar
!

write(*,*) "Enter the identification of the sampled variable:"
write(*,*) "- t (for temperature)"
write(*,*) "- q (for humidity)"
write(*,*) "- oz (for ozone)"
write(*,*) "- ccol (for cloud condensate)"
write(*,*) "- rcol (for precipitation)"
read(*,*) cvar
write(*,*) cvar

if (trim(cvar) .ne. 't'    .and. trim(cvar) .ne. 'q'   .and. trim(cvar) .ne. 'oz' .and. &
    trim(cvar) .ne. 'ccol' .and. trim(cvar) .ne. 'rcol') then
  write(*,*) 'Wrong variable, start again'
  stop
endif

open(1,file='nwp_saf_'//trim(cvar)//'_sampled.atm',form='formatted')
open(2,file='nwp_saf_'//trim(cvar)//'_sampled.sfc',form='formatted')

i = 0
do
  read(1,'(833(e13.6,x),i5,3i3,i7,i5)',end=1) &
     temp(:),     &! 1) Temperature [K]                          (1-91)
     hum(:),      &! 2) Humidity [kg/kg]                         (92-182)
     ozo(:),      &! 3) Ozone [kg/kg]                            (183-273)
     cc(:),       &! 4) Cloud Cover [0-1]                        (274-364)
     clw(:),      &! 5) C Liquid W [kg/kg]                       (365-455)
     ciw(:),      &! 6) C Ice W [kg/kg]                          (456-546)
     rain(:),     &! 7) Rain [kg/(m2 *s)]                        (547-637)
     snow(:),     &! 8) Snow [kg/(m2 *s)]                        (638-728)
     w(:),        &! 9) Vertical Velocity [Pa/s]                 (729-819)
     lnpsurf,     &!10) Ln of Surf Pressure in [Pa]              (820)
     z0,          &!11) Surface geopotential [m2/s2]             (821) 
     tsurf,       &!12) Surface Skin Temperature [K]             (822)
     t2m,         &!13) 2m Temperature [K]                       (823)
     td2m,        &!14) 2m Dew point temperature [K]             (824)
     hum2m,       &!15) 2m Specific Humidity [kg/kg]             (825)
     u10,         &!16) 10m wind speed U component [m/s]         (826)
     v10,         &!17) 10m wind speed V component [m/s]         (827)
     stratrsrf,   &!18) Stratiform rain at surface [kg/(m2 *s)]  (828)
     convrsrf,    &!19) Convective rain at surface [kg/(m2 *s)]  (829)
     snowsurf,    &!20) Snow at surface [kg/(m2 *s)]             (830)
     lsm,         &!21) Land/sea Mask [0-1]                      (831)
     lat,         &!22) Latitude [deg]                           (832)
     long,        &!23) Longitude [deg]                          (833)
     year,        &!24) Year                                     (834)
     month,       &!25) Month                                    (835)
     day,         &!26) Day                                      (836)
     step,        &!27) Step                                     (837)
     gpoint,      &!28) Grid point [1-843490]                    (838)
     ind           !29) Index (rank-sorted)                      (839) 

  read(2,'(26(e13.6,x),i5,3i3,i7,i5)',end=1) &
     alb,         &! 1) Albedo [0-1]                             (1)
     sr,          &! 2) Roughness [m]                            (2)
     tvl,         &! 3) Low  vegetation type [0-20]              (3)
     tvh,         &! 4) High vegetation type [0-19]              (4)
     cvl,         &! 5) Low  vegetation fractional cover [0-1]   (5)
     cvh,         &! 6) High vegetation fractional cover [0-1]   (6)
     seaice,      &! 7) SeaIce cover [0-1]                       (7)
     asn,         &! 8) Snow albedo [0-1]                        (8)
     rsn,         &! 9) Snow density [kg/m3]                     (9)
     tsn,         &!10) Snow temperature [K]                    (10)
     sd,          &!11) Snow depth [m]                          (11)
     stl1,        &!12) Soil Layer 1 temperature [K]            (12)
     stl2,        &!13) Soil Layer 2 temperature [K]            (13)
     stl3,        &!14) Soil Layer 3 temperature [K]            (14)
     stl4,        &!15) Soil Layer 4 temperature [K]            (15)
     swvl1,       &!16) Volumetric Soil Water Layer 1 [m3/m3]   (16)
     swvl2,       &!17) Volumetric Soil Water Layer 2 [m3/m3]   (17)
     swvl3,       &!18) Volumetric Soil Water Layer 3 [m3/m3]   (18)
     swvl4,       &!19) Volumetric Soil Water Layer 4 [m3/m3]   (19)
     istl1,       &!20) Ice temperature Layer 1 [K]             (20)
     istl2,       &!21) Ice temperature Layer 2 [K]             (21)
     istl3,       &!22) Ice temperature Layer 3 [K]             (22)
     istl4,       &!23) Ice temperature Layer 4 [K]             (23)
! The following variables also appear in .atm file
     lsm,         &!%24) Land/sea Mask [0-1]                    (24)
     lat,         &!%25) Latitude [deg]                         (25)
     long,        &!%26) Longitude[deg]                         (26)
     year,        &!%27) Year of start of forecast              (27)
     month,       &!%28) Month of start of forecast             (28)
     day,         &!%29) Day of start of forecast               (29)
     step,        &!%30) Forecast step [hours]                  (30)
     gpoint,      &!%31) Grid point [1-843490]                  (31)
     ind           !%32) Index (rank-sorted)                    (32) 

  ! compute surface pressure [Pa]
  psurf = exp(lnpsurf)

  ! compute surface geometric height [m]
  elev = z0 / 9.80665

  ! compute full-level pressure (pap) and half-level pressures (paph) [Pa]
  ! note that all profile variables above are given at full levels
  call ec_p91l(psurf,pap,paph)

  i = i + 1
enddo
1 write(*,*) 'Number of profiles found in the files:',i

end

! ----------------------------------------
!
      SUBROUTINE EC_P91l(spres,pap,paph)
!
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2006, EUMETSAT, All Rights Reserved.
!
!     Description:
!     Computes the 91-level vertical pressure grid
!       associated to the input surface pressure
!     All pressures are in Pa

      implicit none
      integer, parameter    :: nlev=91
      integer               :: jk
      real      :: spres
      real      :: aam(nlev+1), bbm(nlev+1)
      real      :: pap(nlev), paph(nlev+1)

      data aam / &
         0.000000, &
         2.000040, &
         3.980832, &
         7.387186, &
        12.908319, &
        21.413612, &
        33.952858, &
        51.746601, &
        76.167656, &
       108.715561, &
       150.986023, &
       204.637451, &
       271.356506, &
       352.824493, &
       450.685791, &
       566.519226, &
       701.813354, &
       857.945801, &
      1036.166504, &
      1237.585449, &
      1463.163940, &
      1713.709595, &
      1989.874390, &
      2292.155518, &
      2620.898438, &
      2976.302246, &
      3358.425781, &
      3767.196045, &
      4202.416504, &
      4663.776367, &
      5150.859863, &
      5663.156250, &
      6199.839355, &
      6759.727051, &
      7341.469727, &
      7942.926270, &
      8564.624023, &
      9208.305664, &
      9873.560547, &
     10558.881836, &
     11262.484375, &
     11982.662109, &
     12713.897461, &
     13453.225586, &
     14192.009766, &
     14922.685547, &
     15638.053711, &
     16329.560547, &
     16990.623047, &
     17613.281250, &
     18191.029297, &
     18716.968750, &
     19184.544922, &
     19587.513672, &
     19919.796875, &
     20175.394531, &
     20348.916016, &
     20434.158203, &
     20426.218750, &
     20319.011719, &
     20107.031250, &
     19785.357422, &
     19348.775391, &
     18798.822266, &
     18141.296875, &
     17385.595703, &
     16544.585938, &
     15633.566406, &
     14665.645508, &
     13653.219727, &
     12608.383789, &
     11543.166992, &
     10471.310547, &
      9405.222656, &
      8356.252930, &
      7335.164551, &
      6353.920898, &
      5422.802734, &
      4550.215820, &
      3743.464355, &
      3010.146973, &
      2356.202637, &
      1784.854614, &
      1297.656128, &
       895.193542, &
       576.314148, &
       336.772369, &
       162.043427, &
        54.208336, &
         6.575628, &
         0.003160, &
         0.000000 /

      data bbm / &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000014, &
     0.000055, &
     0.000131, &
     0.000279, &
     0.000548, &
     0.001000, &
     0.001701, &
     0.002765, &
     0.004267, &
     0.006322, &
     0.009035, &
     0.012508, &
     0.016860, &
     0.022189, &
     0.028610, &
     0.036227, &
     0.045146, &
     0.055474, &
     0.067316, &
     0.080777, &
     0.095964, &
     0.112979, &
     0.131935, &
     0.152934, &
     0.176091, &
     0.201520, &
     0.229315, &
     0.259554, &
     0.291993, &
     0.326329, &
     0.362203, &
     0.399205, &
     0.436906, &
     0.475016, &
     0.513280, &
     0.551458, &
     0.589317, &
     0.626559, &
     0.662934, &
     0.698224, &
     0.732224, &
     0.764679, &
     0.795385, &
     0.824185, &
     0.850950, &
     0.875518, &
     0.897767, &
     0.917651, &
     0.935157, &
     0.950274, &
     0.963007, &
     0.973466, &
     0.982238, &
     0.989153, &
     0.994204, &
     0.997630, &
     1.000000 /

      do jk=1,nlev+1
      paph(jk)=aam(jk)+bbm(jk)*spres
      end do
      do jk=1,nlev
      pap(jk)=0.5*(paph(jk)+paph(jk+1))
      end do

      return
      end subroutine ec_p91l
